@@ -69,6 +69,7 @@ gcpAtom::gcpAtom(): Atom()
 	m_ChargePos = 0xff;
 	m_ChargeAngle = 0.;
 	m_ChargeDist = 0.;
+	m_ChargeAutoPos = true;
 }
 
 gcpAtom::~gcpAtom()
@@ -101,6 +102,7 @@ gcpAtom::gcpAtom(int Z, double x, double y, double z): Atom(Z, x, y, z)
 	m_ChargePos = 0xff;
 	m_ChargeAngle = 0.;
 	m_ChargeDist = 0.;
+	m_ChargeAutoPos = true;
 }
 
 gcpAtom::gcpAtom(OBAtom* atom, double zoom): Atom()
@@ -124,6 +126,7 @@ gcpAtom::gcpAtom(OBAtom* atom, double zoom): Atom()
 	m_ChargePos = 0xff;
 	m_ChargeAngle = 0.;
 	m_ChargeDist = 0.;
+	m_ChargeAutoPos = true;
 }
 
 void gcpAtom::SetZ(int Z)
@@ -236,10 +239,12 @@ void gcpAtom::Update()
 			m_ChargeAuto = true;
 			m_nH = 0;
 		}
-		if (m_nH < 0) { // extended octet
+		if (m_nH < 0) { // extended octet or missing core electrons
 			m_nH = 0;
-			m_Charge = m_Element->GetValenceElectrons () - 2 * m_nlp - nexplu - nbonds;
-			m_ChargeAuto = true;
+			if (m_nlp || nexplu || nbonds) {
+				m_Charge = m_Element->GetValenceElectrons () - 2 * m_nlp - nexplu - nbonds;
+				m_ChargeAuto = true;
+			}
 		}
 		m_HPos = GetBestSide();
 	} else {
@@ -317,15 +322,15 @@ void gcpAtom::Add(GtkWidget* w)
 		gtk_text_tag_table_add(TextTagTable, tag);
 		g_object_unref((GObject*)tag);
 	}
-	double x, y;
+	double x, y, xc, yc;
 	m_width =  m_height = 2.0 * pData->Padding;
 	GetCoords(&x, &y);
 	x *= pData->ZoomFactor;
 	y *= pData->ZoomFactor;
 	double dFontHeight = pData->View->GetFontHeight();
 	char * szFontName = pData->View->GetFontName();
-	GnomeCanvasItem* item;
-	GnomeCanvasGroup* group;
+	GnomeCanvasItem *item;
+	GnomeCanvasGroup *group, *chgp;
 	int index;
 	char *hs;
 	PangoContext* pc = pData->View->GetPangoContext();
@@ -433,7 +438,7 @@ void gcpAtom::Add(GtkWidget* w)
 	/* add charge */
 	int charge = GetCharge ();
 	if (charge) {
-		int align = GetChargePosition(m_ChargePos, m_ChargeAngle, x, y);
+		int align = GetChargePosition (m_ChargePos, m_ChargeAngle * 180 / M_PI, x, y);
 		if (m_ChargeDist != 0.) {
 			align = 0;
 			x = m_x + m_ChargeDist * cos (m_ChargeAngle);
@@ -441,46 +446,101 @@ void gcpAtom::Add(GtkWidget* w)
 		}
 		x *= pData->ZoomFactor;
 		y *= pData->ZoomFactor;
-		GtkAnchorType anchor;
-		switch (align)
-		{
-			case -2:
-				y += 6.0;
-				anchor = GTK_ANCHOR_NORTH;
-				break;
-			case -1:
-				x -= 6.0;
-				anchor = GTK_ANCHOR_EAST;
-				break;
-			case 0:
-				anchor = GTK_ANCHOR_CENTER;
-				break;
-			case 1:
-				x += 6.0;
-				anchor = GTK_ANCHOR_WEST;
-				break;
-			case 2:
-				y -= 6.0;
-				anchor = GTK_ANCHOR_SOUTH;
-				break;
+		char *fig = NULL;
+		if 	(abs (charge) > 1) {
+			fig = g_strdup_printf ("%d", abs (charge));
+			PangoContext* pc = pData->View->GetPangoContext();
+			PangoRectangle rect;
+			pango_context_set_font_description (pc, pData->View->GetPangoSmallFontDesc ());
+			PangoLayout* pl = pango_layout_new (pc);
+			pango_layout_set_text (pl, fig, -1);
+			pango_layout_get_extents (pl, NULL, &rect);
+			m_ChargeWidth = rect.width / PANGO_SCALE;
+			pango_context_set_font_description (pc, pData->View->GetPangoFontDesc ());
+			m_ChargeTWidth = m_ChargeWidth + 1. + ChargeSignSize;
+		} else {
+			m_ChargeWidth = 0.;
+			m_ChargeTWidth = ChargeSignSize;
 		}
-		char *markup;
-		if 	(abs (charge) > 1)
-			markup = g_strdup_printf ("%d<span size=\"larger\">%s</span>", abs (charge), (charge > 0)? "\xE2\x8a\x95": "\xE2\x8a\x96");
-		else
-			markup = g_strdup_printf ("<span size=\"larger\">%s</span>", (charge > 0)? "\xE2\x8a\x95": "\xE2\x8a\x96");
-		item = gnome_canvas_item_new(
-						group,
+		switch (align) {
+		case -2:
+			xc = x + m_ChargeTWidth / 2. - ChargeSignSize;
+			y += ChargeSignHeight / 2.;
+			break;
+		case -1:
+			xc = x - ChargeSignSize -1.;
+			break;
+		case 0:
+		case -3:
+			xc = x + m_ChargeTWidth / 2. - ChargeSignSize;
+			break;
+		case 1:
+			xc = x + m_ChargeWidth + 1.;
+			break;
+		case 2:
+			xc = x + m_ChargeTWidth / 2. - ChargeSignSize;
+			y -= ChargeSignHeight / 2.;
+			break;
+		}
+		x = xc - 1.;
+		yc = y - ChargeSignHeight / 2. + ChargeYAlign - ChargeSignSize + 1.;
+		chgp = (GnomeCanvasGroup*) gnome_canvas_item_new (
+					group,
+					gnome_canvas_group_ext_get_type(),
+					NULL);
+		g_object_set_data (G_OBJECT (group), "charge", chgp);
+		if (fig) {
+			item = gnome_canvas_item_new(
+						chgp,
 						gnome_canvas_text_ext_get_type(),
 						"fill_color", (pData->IsSelected(this))? SelectColor: Color,
 						"font-desc", pData->View->GetPangoSmallFontDesc (),
-						"markup", markup,
-						"anchor", anchor,
+						"markup", fig,
+						"anchor", GTK_ANCHOR_EAST,
 						"x", x,
 						"y", y,
 						NULL);
-		g_free (markup);
-		g_object_set_data ((GObject*)group, "charge", item);
+			g_object_set_data (G_OBJECT (group), "figure", item);
+			g_free (fig);
+		}
+		item = gnome_canvas_item_new (
+					chgp,
+					gnome_canvas_ellipse_ext_get_type (),
+					"x1", xc,
+					"y1", yc,
+					"x2", xc + ChargeSignSize,
+					"y2", yc + ChargeSignSize,
+					"outline_color", (pData->IsSelected(this))? SelectColor: Color,
+					"width_units", 0.5,
+					NULL
+				);
+		g_object_set_data (G_OBJECT (group), "circle", item);
+		ArtBpath *path = art_new (ArtBpath, 5);
+		path[0].code = ART_MOVETO_OPEN;
+		path[0].x3 = xc + 1.;
+		path[1].code = ART_LINETO;
+		path[1].x3 = xc + ChargeSignSize - 1.;
+		path[0].y3 = path[1].y3 = yc + ChargeSignSize / 2.;
+		if (charge > 0) {
+			path[2].code = ART_MOVETO_OPEN;
+			path[2].y3 = yc + 1.;
+			path[3].code = ART_LINETO;
+			path[3].y3 = yc + ChargeSignSize - 1.;
+			path[2].x3 = path[3].x3 = xc + ChargeSignSize / 2.;
+			path[4].code = ART_END;
+		} else
+			path[2].code = ART_END;
+		GnomeCanvasPathDef *cpd = gnome_canvas_path_def_new_from_bpath (path);
+		item = gnome_canvas_item_new (
+					chgp,
+					gnome_canvas_bpath_ext_get_type (),
+					"bpath", cpd,
+					"outline_color", (pData->IsSelected(this))? SelectColor: Color,
+					"width_units", 1.,
+					NULL
+				);
+		gnome_canvas_path_def_unref (cpd);
+		g_object_set_data (G_OBJECT (group), "sign", item);
 	}
 	map<string, Object*>::iterator i;
 	Object* electron = GetFirstChild (i);
@@ -494,7 +554,7 @@ void gcpAtom::Update(GtkWidget* w)
 {
 	if (!w) return;
 	gcpWidgetData* pData = (gcpWidgetData*)g_object_get_data(G_OBJECT(w), "data");
-	double x, y;
+	double x, y, xc, yc;
 	GetCoords(&x, &y);
 	x *= pData->ZoomFactor;
 	y *= pData->ZoomFactor;
@@ -521,11 +581,11 @@ void gcpAtom::Update(GtkWidget* w)
 									"y2", y + 3,
 								NULL);
 	}
-	void* item = g_object_get_data(G_OBJECT(group), "charge");
+	void* item = g_object_get_data (G_OBJECT (group), "charge");
 	int charge = GetCharge ();
 	if (charge) {
 		if (item) {
-			int align = GetChargePosition(m_ChargePos, m_ChargeAngle, x, y);
+			int align = GetChargePosition(m_ChargePos, m_ChargeAngle * 180 / M_PI, x, y);
 			if (m_ChargeDist != 0.) {
 				align = 0;
 				x = m_x + m_ChargeDist * cos (m_ChargeAngle);
@@ -533,39 +593,96 @@ void gcpAtom::Update(GtkWidget* w)
 			}
 			x *= pData->ZoomFactor;
 			y *= pData->ZoomFactor;
-			GtkAnchorType anchor;
-			switch (align)
-			{
-				case -2:
-					anchor = GTK_ANCHOR_NORTH;
-					break;
-				case -1:
-					anchor = GTK_ANCHOR_EAST;
-					break;
-				case 0:
-					anchor = GTK_ANCHOR_CENTER;
-					break;
-				case 1:
-					anchor = GTK_ANCHOR_WEST;
-					break;
-				case 2:
-					anchor = GTK_ANCHOR_SOUTH;
-					break;
+			GnomeCanvasItem *figure = (GnomeCanvasItem*) g_object_get_data (G_OBJECT (group), "figure");
+			char *fig = NULL;
+			if 	(abs (charge) > 1) {
+				fig = g_strdup_printf ("%d", abs (charge));
+				PangoContext* pc = pData->View->GetPangoContext();
+				PangoRectangle rect;
+				pango_context_set_font_description (pc, pData->View->GetPangoSmallFontDesc ());
+				PangoLayout* pl = pango_layout_new (pc);
+				pango_layout_set_text (pl, fig, -1);
+				pango_layout_get_extents (pl, NULL, &rect);
+				m_ChargeWidth = rect.width / PANGO_SCALE;
+				pango_context_set_font_description (pc, pData->View->GetPangoFontDesc ());
+			} else
+				m_ChargeWidth = 0.;
+			m_ChargeTWidth = m_ChargeWidth + 1. + ChargeSignSize;
+			if (figure == NULL && fig != NULL) {
+				figure = gnome_canvas_item_new(
+							GNOME_CANVAS_GROUP (item),
+							gnome_canvas_text_ext_get_type(),
+							"fill_color", (pData->IsSelected(this))? SelectColor: Color,
+							"font-desc", pData->View->GetPangoSmallFontDesc (),
+							"anchor", GTK_ANCHOR_EAST,
+							NULL);
+				g_object_set_data (G_OBJECT (group), "figure", figure);
+			} else if (figure != NULL && fig == NULL) {
+				gtk_object_destroy (GTK_OBJECT (figure));
+				g_object_set_data ((GObject*)group, "figure", NULL);
 			}
-			char *markup;
-			if 	(abs (charge) > 1)
-				markup = g_strdup_printf ("%d<span size=\"larger\">%s</span>", abs (charge), (charge > 0)? "\xE2\x8a\x95": "\xE2\x8a\x96");
-			else
-				markup = g_strdup_printf ("<span size=\"larger\">%s</span>", (charge > 0)? "\xE2\x8a\x95": "\xE2\x8a\x96");
+			switch (align) {
+			case -2:
+				xc = x + m_ChargeTWidth / 2. - ChargeSignSize;
+				y += ChargeSignHeight / 2.;
+				break;
+			case -1:
+				xc = x - ChargeSignSize -1.;
+				break;
+			case 0:
+			case -3:
+				xc = x + m_ChargeTWidth / 2. - ChargeSignSize;
+				break;
+			case 1:
+				xc = x + m_ChargeWidth + 1.;
+				break;
+			case 2:
+				xc = x + m_ChargeTWidth / 2. - ChargeSignSize;
+				y -= ChargeSignHeight / 2.;
+				break;
+			}
+			x = xc - 1.;
+			yc = y - ChargeSignHeight / 2. + ChargeYAlign - ChargeSignSize + 1.;
+			if (fig) {
+				g_object_set (G_OBJECT (figure),
+							"markup", fig,
+							"x", x,
+							"y", y,
+							NULL);
+				g_free (fig);
+			}
+			item = g_object_get_data (G_OBJECT (group), "circle");
 			g_object_set (G_OBJECT (item),
-						"markup", markup,
-						"anchor", anchor,
-						"x", x,
-						"y", y,
+						"x1", xc,
+						"y1", yc,
+						"x2", xc + ChargeSignSize,
+						"y2", yc + ChargeSignSize,
 						NULL);
-			g_free (markup);
+			item = g_object_get_data (G_OBJECT (group), "sign");
+			ArtBpath *path = art_new (ArtBpath, 5);
+			path[0].code = ART_MOVETO_OPEN;
+			path[0].x3 = xc + 1.;
+			path[1].code = ART_LINETO;
+			path[1].x3 = xc + ChargeSignSize - 1.;
+			path[0].y3 = path[1].y3 = yc + ChargeSignSize / 2.;
+			if (charge > 0) {
+				path[2].code = ART_MOVETO_OPEN;
+				path[2].y3 = yc + 1.;
+				path[3].code = ART_LINETO;
+				path[3].y3 = yc + ChargeSignSize - 1.;
+				path[2].x3 = path[3].x3 = xc + ChargeSignSize / 2.;
+				path[4].code = ART_END;
+			} else
+				path[2].code = ART_END;
+			GnomeCanvasPathDef *cpd = gnome_canvas_path_def_new_from_bpath (path);
+			g_object_set (G_OBJECT (item),
+						"bpath", cpd,
+						NULL
+					);
+			gnome_canvas_path_def_unref (cpd);
 		} else {
-			int align = GetChargePosition(m_ChargePos, m_ChargeAngle, x, y);
+			GnomeCanvasGroup *chgp;
+			int align = GetChargePosition(m_ChargePos, m_ChargeAngle * 180 / M_PI, x, y);
 			if (m_ChargeDist != 0.) {
 				align = 0;
 				x = (m_x + m_ChargeDist * cos (m_ChargeAngle));
@@ -574,44 +691,102 @@ void gcpAtom::Update(GtkWidget* w)
 			x *= pData->ZoomFactor;
 			y *= pData->ZoomFactor;
 
-			GtkAnchorType anchor;
-			switch (align)
-			{
-				case -2:
-					anchor = GTK_ANCHOR_NORTH;
-					break;
-				case -1:
-					anchor = GTK_ANCHOR_EAST;
-					break;
-				case 0:
-					anchor = GTK_ANCHOR_CENTER;
-					break;
-				case 1:
-					anchor = GTK_ANCHOR_WEST;
-					break;
-				case 2:
-					anchor = GTK_ANCHOR_SOUTH;
-					break;
+			char *fig = NULL;
+			if 	(abs (charge) > 1) {
+				fig = g_strdup_printf ("%d", abs (charge));
+				PangoContext* pc = pData->View->GetPangoContext();
+				PangoRectangle rect;
+				pango_context_set_font_description (pc, pData->View->GetPangoSmallFontDesc ());
+				PangoLayout* pl = pango_layout_new (pc);
+				pango_layout_set_text (pl, fig, -1);
+				pango_layout_get_extents (pl, NULL, &rect);
+				m_ChargeWidth = rect.width / PANGO_SCALE;
+				pango_context_set_font_description (pc, pData->View->GetPangoFontDesc ());
+				m_ChargeTWidth = m_ChargeWidth + 2 + ChargeSignSize;
+			} else {
+				m_ChargeWidth = 0.;
+				m_ChargeTWidth = ChargeSignSize;
 			}
-			char *markup;
-			if 	(abs (charge) > 1)
-				markup = g_strdup_printf ("%d<span size=\"larger\">%s</span>", abs (charge), (charge > 0)? "\xE2\x8a\x95": "\xE2\x8a\x96");
-			else
-				markup = g_strdup_printf ("<span size=\"larger\">%s</span>", (charge > 0)? "\xE2\x8a\x95": "\xE2\x8a\x96");
-			item = gnome_canvas_item_new(
-							group,
+			switch (align) {
+			case -2:
+				xc = x + m_ChargeTWidth / 2. - ChargeSignSize;
+				y += ChargeSignHeight / 2.;
+				break;
+			case -1:
+				xc = x - ChargeSignSize -1.;
+				break;
+			case 0:
+			case -3:
+				xc = x + m_ChargeTWidth / 2. - ChargeSignSize;
+				break;
+			case 1:
+				xc = x + m_ChargeWidth + 1.;
+				break;
+			case 2:
+				xc = x + m_ChargeTWidth / 2. - ChargeSignSize;
+				y -= ChargeSignHeight / 2.;
+				break;
+			}
+			x = xc - 1.;
+			yc = y - ChargeSignHeight / 2. + ChargeYAlign - ChargeSignSize + 1.;
+			chgp = (GnomeCanvasGroup*) gnome_canvas_item_new (
+						group,
+						gnome_canvas_group_ext_get_type(),
+						NULL);
+			g_object_set_data (G_OBJECT (group), "charge", chgp);
+			if (fig) {
+				item = gnome_canvas_item_new(
+							chgp,
 							gnome_canvas_text_ext_get_type(),
 							"fill_color", (pData->IsSelected(this))? SelectColor: Color,
 							"font-desc", pData->View->GetPangoSmallFontDesc (),
-							"markup", markup,
-							"anchor", anchor,
+							"markup", fig,
+							"anchor", GTK_ANCHOR_EAST,
 							"x", x,
 							"y", y,
 							NULL);
-			g_free (markup);
-			g_object_set_data ((GObject*)group, "charge", item);
+				g_object_set_data (G_OBJECT (group), "figure", item);
+				g_free (fig);
+			}
+			item = gnome_canvas_item_new (
+						chgp,
+						gnome_canvas_ellipse_ext_get_type (),
+						"x1", xc,
+						"y1", yc,
+						"x2", xc + ChargeSignSize,
+						"y2", yc + ChargeSignSize,
+						"outline_color", (pData->IsSelected(this))? SelectColor: Color,
+						"width_units", .5,
+						NULL
+					);
+			g_object_set_data (G_OBJECT (group), "circle", item);
+			ArtBpath *path = art_new (ArtBpath, 5);
+			path[0].code = ART_MOVETO_OPEN;
+			path[0].x3 = xc + 1.;
+			path[1].code = ART_LINETO;
+			path[1].x3 = xc + ChargeSignSize - 1.;
+			path[0].y3 = path[1].y3 = yc + ChargeSignSize / 2.;
+			if (charge > 0) {
+				path[2].code = ART_MOVETO_OPEN;
+				path[2].y3 = yc + 1.;
+				path[3].code = ART_LINETO;
+				path[3].y3 = yc + ChargeSignSize - 1.;
+				path[2].x3 = path[3].x3 = xc + ChargeSignSize / 2.;
+				path[4].code = ART_END;
+			} else
+				path[2].code = ART_END;
+			GnomeCanvasPathDef *cpd = gnome_canvas_path_def_new_from_bpath (path);
+			item = gnome_canvas_item_new (
+						chgp,
+						gnome_canvas_bpath_ext_get_type (),
+						"bpath", cpd,
+						"outline_color", (pData->IsSelected(this))? SelectColor: Color,
+						"width_units", 1.,
+						NULL
+					);
+			gnome_canvas_path_def_unref (cpd);
+			g_object_set_data (G_OBJECT (group), "sign", item);
 		}
-		gnome_canvas_item_show (GNOME_CANVAS_ITEM (item));
 	} else {
 		if (item)
 			gtk_object_destroy (GTK_OBJECT (item));
@@ -685,9 +860,17 @@ int gcpAtom::GetChargePosition(unsigned char& Pos, double Angle, double& x, doub
 {
 	list<double>::iterator n;
 	double angle;
+	if (m_ChargePos != 0xff)
+		m_OccupiedPos &= ~m_ChargePos;
 	if (!m_AvailPosCached)
 		UpdateAvailablePositions ();
-	if (Pos == 0xff) {
+	if (m_ChargePos != 0xff)
+		m_OccupiedPos |= m_ChargePos;
+	if (!m_ChargeAutoPos && Pos == 0xff) {
+		Pos = m_ChargePos;
+		if (!Pos)
+			Angle = m_ChargeAngle * 180 / M_PI;
+	} else if (Pos == 0xff) {
 		if (m_AvailPos) {
 			if (m_AvailPos & CHARGE_NE)
 				Pos = CHARGE_NE;
@@ -910,8 +1093,12 @@ void gcpAtom::SetSelected(GtkWidget* w, int state)
 	}
 	g_object_set (G_OBJECT(g_object_get_data(G_OBJECT(group), "rect")),
 				"fill_color", color, NULL);
-	if (item = g_object_get_data(G_OBJECT(group), "charge"))
+	if (item = g_object_get_data(G_OBJECT(group), "figure"))
 		g_object_set (item, "fill_color", chargecolor, NULL);
+	if (item = g_object_get_data(G_OBJECT(group), "circle"))
+		g_object_set (item, "outline_color", chargecolor, NULL);
+	if (item = g_object_get_data(G_OBJECT(group), "sign"))
+		g_object_set (item, "outline_color", chargecolor, NULL);
 	Object::SetSelected (w, state);
 }
 
@@ -1261,6 +1448,7 @@ bool gcpAtom::Load (xmlNodePtr node)
 			m_ChargePos = CHARGE_W;
 			m_ChargeAngle = M_PI;
 		}
+		m_ChargeAutoPos = false;
 		xmlFree (buf);
 	} else {
 		buf = (char*) xmlGetProp(node, (xmlChar*)"charge-angle");
@@ -1269,12 +1457,14 @@ bool gcpAtom::Load (xmlNodePtr node)
 			m_ChargeAngle *= M_PI / 180.;
 			xmlFree (buf);
 			m_ChargePos = 0;
+			m_ChargeAutoPos = false;
 		}
 	}
 	buf = (char*) xmlGetProp(node, (xmlChar*)"charge-dist");
 	if (buf) {
 		sscanf(buf, "%lg", &m_ChargeDist);
 		xmlFree (buf);
+		m_ChargeAutoPos = false;
 	} else
 		m_ChargeDist = 0.;
 	return true;
@@ -1300,12 +1490,14 @@ bool gcpAtom::AcceptCharge (int charge) {
 
 void gcpAtom::SetChargePosition (unsigned char Pos, bool def, double angle, double distance)
 {
-	m_ChargeAutoPos = def;
-	if (m_ChargePos > 0)
-		NotifyPositionOccupation (m_ChargePos, false);
-	m_ChargePos = Pos;
-	if (m_ChargePos > 0)
-		NotifyPositionOccupation (m_ChargePos, true);
+	if (Pos != m_ChargePos) {
+		m_ChargeAutoPos = def;
+		if (m_ChargePos > 0)
+			NotifyPositionOccupation (m_ChargePos, false);
+		m_ChargePos = Pos;
+		if (m_ChargePos > 0)
+			NotifyPositionOccupation (m_ChargePos, true);
+	}
 	m_ChargeAngle = angle;
 	m_ChargeDist = distance;
 	m_AvailPosCached = false;
@@ -1319,3 +1511,10 @@ char gcpAtom::GetChargePosition (double *Angle, double *Dist)
 		*Dist = m_ChargeDist;
 	return (m_ChargeAutoPos)? -1: m_ChargePos;
 }
+
+void gcpAtom::SetCharge (int charge)
+{
+	gcu::Atom::SetCharge (charge);
+	m_ChargeAuto = false;
+	Update ();
+}
