@@ -18,7 +18,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 
@@ -112,7 +112,7 @@ void gcpGroup::Align ()
 	double x, t;
 	if ((m_AlignType == GCP_ALIGN_TOP) 
 		|| (m_AlignType == GCP_ALIGN_LEFT))
-		t = INFINITY;
+		t = DBL_MAX;
 	gcpWidgetData *Data = (gcpWidgetData*) g_object_get_data (G_OBJECT (View->GetWidget ()), "data");
 	while (obj) {
 		if (m_AlignType == GCP_ALIGN_NORMAL) {
@@ -321,3 +321,16 @@ bool gcpGroup::OnSignal (SignalId Signal, Object *Child)
 	}
 	return true;
 }
+
+void gcpGroup::Transform2D (Matrix2D& m, double x, double y)
+{
+}
+
+double gcpGroup::GetYAlign ()
+{
+	gcpDocument *pDoc = reinterpret_cast<gcpDocument*> (GetDocument ());
+	gcpWidgetData* pData = (gcpWidgetData*) g_object_get_data (G_OBJECT (pDoc->GetWidget ()), "data");
+	ArtDRect rect;
+	pData->GetObjectBounds (this, &rect);
+	return (rect.y1 - rect.y0) / 2.;
+}
