@@ -20,7 +20,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  */
 
@@ -381,10 +381,6 @@ static const char *ui_web_description =
 "  </menubar>"
 "</ui>";
 
-#warning "the following lines should be removed for stable releases"
-#undef PACKAGE
-#define PACKAGE "gchempaint-unstable" 
-
 gcpStandaloneApp::gcpStandaloneApp(): gcpApplication()
 {
 	GtkWidget *vbox;
@@ -399,7 +395,7 @@ gcpStandaloneApp::gcpStandaloneApp(): gcpApplication()
 	m_MessageId = 0;
 	m_Window = GTK_WINDOW (gtk_window_new (GTK_WINDOW_TOPLEVEL));
 	gtk_window_set_title (m_Window, _("GChemPaint"));
-	gtk_window_set_icon_from_file (m_Window, PIXMAPSDIR"/gchempaint.png", NULL);
+	gtk_window_set_icon_name (m_Window, "gchempaint");
 	g_signal_connect(G_OBJECT(m_Window), "destroy", G_CALLBACK(on_quit), this);
 	g_signal_connect(G_OBJECT(m_Window), "focus_in_event", G_CALLBACK(on_focus), this);
 	vbox = gtk_vbox_new (FALSE, 0);
@@ -526,7 +522,7 @@ void gcpStandaloneApp::OnFileNew()
 {
 	gchar tmp[32];
 	if (m_pActiveDoc && !m_pActiveDoc->GetView()->PrepareUnselect()) return;
-	g_snprintf(tmp, sizeof(tmp), _("Untitled%d"), m_NumWindow++);
+	g_snprintf(tmp, sizeof(tmp), _("Untitled %d"), m_NumWindow++);
 	GtkWidget* w = CreateView();
 	if (w)
 	{
@@ -553,7 +549,7 @@ void gcpStandaloneApp::OnFileNew()
 		gtk_scrolled_window_set_policy(scroll, GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 		gtk_scrolled_window_set_shadow_type (scroll, GTK_SHADOW_IN);
 		gtk_scrolled_window_add_with_viewport(scroll, w);
-		gtk_widget_set_size_request(GTK_WIDGET(scroll), 400, 300);
+		gtk_widget_set_size_request(GTK_WIDGET(scroll), 408, 308);
 		gtk_widget_show(GTK_WIDGET(scroll));
 		gtk_notebook_append_page(m_Book, GTK_WIDGET(scroll), label);
 		gtk_notebook_set_current_page(m_Book, g_list_length(m_Book->children) - 1);
@@ -602,7 +598,7 @@ void  gcpStandaloneApp::AddDocument(gcpDocument* pDoc)
 		gtk_scrolled_window_set_policy(scroll, GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
 		gtk_scrolled_window_set_shadow_type (scroll, GTK_SHADOW_IN);
 		gtk_scrolled_window_add_with_viewport(scroll, w);
-		gtk_widget_set_size_request(GTK_WIDGET(scroll), 400, 300);
+		gtk_widget_set_size_request(GTK_WIDGET(scroll), 408, 308);
 		gtk_widget_show(GTK_WIDGET(scroll));
 		gtk_notebook_append_page(m_Book, GTK_WIDGET(scroll), label);
 		gtk_notebook_set_current_page(m_Book, g_list_length(m_Book->children) - 1);
@@ -848,6 +844,7 @@ void gcpStandaloneApp::OnPrint()
 	int do_preview, copies = 1, collate = 0;
 	GnomePrintDialog *gpd;
 	gpd = GNOME_PRINT_DIALOG (gnome_print_dialog_new(gpj, (const guchar*)_("Print"), GNOME_PRINT_DIALOG_COPIES));
+	gtk_window_set_icon_name (GTK_WINDOW (gpd), "gchempaint");
 	gnome_print_dialog_set_copies(gpd, copies, collate);
 	switch (gtk_dialog_run(GTK_DIALOG(gpd)))
 	{
@@ -872,7 +869,9 @@ void gcpStandaloneApp::OnPrint()
 	gnome_print_job_close(gpj);
 	if (do_preview)
 	{
-		gtk_widget_show (gnome_print_job_preview_new (gpj, (const guchar*)_("Preview")));
+		GtkWidget *preview = gnome_print_job_preview_new (gpj, (const guchar*)_("Preview"));
+		gtk_window_set_icon_name (GTK_WINDOW (preview), "gchempaint");
+		gtk_widget_show (preview);
 	}
 	else
 	{
